#ifndef NODE_H
#define NODE_H

#include <stdbool.h>
#include <stddef.h>

#include <stc/cstr.h>
#include <stc/forward.h>
typedef struct Node Node;
forward_cvec(Nodes, struct Node);
forward_cmap(NodeMap, cstr, struct Node);

typedef enum {
    NODE_FUNCTION_CALL = 'C',
    NODE_FUNCTION_DECL = 'D',
    NODE_FUNCTION_IMPL = 'I',
    NODE_ARG_DECL = 'A',
    NODE_RETURN = 'R',
    NODE_REFERENCE = 'r',
    NODE_NUMBER = 'N',
} NodeType;

typedef struct {
    const char* name;
    Nodes args;
} FunctionCall;

typedef struct {
    const char* type;
    const char* name;
    Nodes args;
} FunctionDecl;

typedef struct {
    const char* type;
    const char* name;
    Nodes args;
    Nodes body;
} FunctionImpl;

typedef struct {
    const char* type;
    const char* name;
} ArgDecl;

typedef struct {
    const char* type;
    const char* name;
} Reference;

typedef struct {
    int value;
} Number;

typedef struct {
    Node* value;
} Return;

struct Nodes;
typedef struct Node {
    NodeType type;
    union {
        FunctionCall function_call;
        FunctionDecl function_decl;
        FunctionImpl function_impl;
        ArgDecl arg_decl;
        Reference reference;
        Number number;
        Return ret;
    };
} Node;

#define i_type Nodes
#define i_is_forward
#define i_val Node
#define i_opt c_no_cmp
#include <stc/cvec.h>

#define i_type NodeMap
#define i_is_forward
#define i_key_str
#define i_val Node
#include <stc/cmap.h>

#endif
